"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const graphql_executor_1 = require("./graphql-executor");
const LRU = require("lru-cache");
const debug = require("debug")("gamebot:api-client");
class CacheGraphQlQueryExecutor extends graphql_executor_1.GraphQlQueryExecutor {
    constructor(request, cacheOptions) {
        super(request);
        this.storage = {};
        for (let key of Object.keys(cacheOptions)) {
            this.storage[key] = new LRU({
                max: cacheOptions[key].max,
                maxAge: cacheOptions[key].ttl
            });
        }
    }
    execute(type, items) {
        const _super = name => super[name];
        return __awaiter(this, void 0, void 0, function* () {
            if (type !== "query") {
                return _super("execute").call(this, type, items);
            }
            const result = { data: {} };
            const nonCacheItems = {};
            const keys = Object.keys(items);
            let hasNonCacheItems = false;
            for (let key of keys) {
                const item = items[key];
                const cache = this.storage[item.name];
                if (cache) {
                    const itemKey = this.createItemKey(item);
                    const value = cache.get(itemKey);
                    if (value !== undefined) {
                        result.data[key] = value;
                        continue;
                    }
                }
                nonCacheItems[key] = item;
                hasNonCacheItems = true;
            }
            if (!hasNonCacheItems) {
                debug(`none non-cache items!`);
                return result;
            }
            const remoteResult = yield _super("execute").call(this, type, nonCacheItems);
            if (remoteResult.data) {
                this.setCacheData(remoteResult.data, nonCacheItems);
            }
            result.data = Object.assign(result.data, remoteResult.data);
            if (remoteResult.errors) {
                result.errors = remoteResult.errors;
            }
            return result;
        });
    }
    setCacheData(data, items) {
        for (let key in data) {
            const item = items[key];
            const cache = this.storage[item.name];
            if (!cache) {
                continue;
            }
            const value = data[key];
            if (value !== undefined) {
                const itemKey = this.createItemKey(item);
                cache.set(itemKey, value);
            }
        }
    }
    createItemKey(item) {
        return (item.name +
            "#" +
            (item.variables
                ? JSON.stringify(item.variables.map(item => item.value))
                : "") +
            "#" +
            item.fields || "");
    }
}
exports.CacheGraphQlQueryExecutor = CacheGraphQlQueryExecutor;
