"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const program = require("commander");
const schema_1 = require("./schema");
const types_generator_1 = require("./types-generator");
const path_1 = require("path");
const api_generator_1 = require("./api-generator");
const client_generator_1 = require("./client-generator");
program
    .version("0.1.0")
    .option("-o, --output <value>", "Output path")
    .option("-p, --endpoint <value>", "Api endpoint")
    .option("-h, --headers [value]", "Headers", h => JSON.parse(h))
    .parse(process.argv);
const output = program.output || process.cwd();
const endpoint = program.endpoint;
const headers = program.headers;
if (!endpoint) {
    throw new Error(`--endpoint is required!`);
}
function run() {
    return __awaiter(this, void 0, void 0, function* () {
        console.log(`output: ${output}`);
        console.log(`endpoint: ${endpoint}`);
        console.log(`headers: ${headers}`);
        const schemaData = yield schema_1.SchemaData.create(endpoint, headers);
        const typesFilename = "api-types";
        const typesGenerator = new types_generator_1.TypesGenerator(schemaData, path_1.join(output, typesFilename + ".ts"));
        const queryApiFilename = "query-api";
        const queryApiGenerator = new api_generator_1.ApiGenerator(schemaData, path_1.join(output, queryApiFilename + ".ts"), "query", typesFilename);
        const mutationApiFilename = "mutation-api";
        const mutationApiGenerator = new api_generator_1.ApiGenerator(schemaData, path_1.join(output, mutationApiFilename + ".ts"), "mutation", typesFilename);
        const clientFilename = "graphql-client";
        const clientGenerator = new client_generator_1.ClientGenerator(path_1.join(output, clientFilename + ".ts"), queryApiFilename, mutationApiFilename);
        yield typesGenerator.generate();
        yield queryApiGenerator.generate();
        yield mutationApiGenerator.generate();
        yield clientGenerator.generate();
    });
}
run()
    .then(() => console.log("OK!"))
    .catch(e => console.trace(e));
