"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const schema_1 = require("./schema");
const generator_1 = require("./generator");
class TypesGenerator extends generator_1.FileGenerator {
    constructor(schemaData, file) {
        super(file);
        this.schemaData = schemaData;
    }
    getContent() {
        return __awaiter(this, void 0, void 0, function* () {
            const schema = this.schemaData.schema;
            const typeNames = schema.data.__schema.types
                .map((t) => this.schemaData.getTypeByName(t.name))
                .filter(type => this.schemaData.typeIsObject(type));
            const types = typeNames.map(name => this.schemaData.getTypeByName(name.name));
            let data = types
                .map(item => ({
                name: item.name,
                fields: (item.fields || item.inputFields).map((field) => ({
                    name: field.name,
                    type: schema_1.getJsTypeName(field.type),
                    required: schema_1.typeIsRequired(field.type)
                }))
            }))
                .map(item => `export type ${item.name} = ${schema_1.generateTypeScriptType(item.fields)}`);
            data = data.concat(types.map(type => `export const ${type.name}StringFields = '${this.getTypeStringFields(type)}';`));
            const enumNames = schema.data.__schema.types
                .map((t) => this.schemaData.getTypeByName(t.name))
                .filter(type => this.schemaData.typeIsEnum(type));
            const enumTypes = enumNames.map(name => this.schemaData.getTypeByName(name.name));
            const enums = enumTypes.map(item => `export enum ${item.name} ${schema_1.generateEnumScript(item.enumValues)}`);
            data = data.concat(enums);
            return data.join("\n\n");
        });
    }
    getTypeStringFields(type, parent = {}) {
        const fields = type.fields || type.inputFields;
        return fields
            .map(field => {
            const fieldsType = this.schemaData.getTypeByName(schema_1.getTypeName(field.type));
            const fieldKey = `${field.name}-${fieldsType.name}`;
            const isObject = this.schemaData.typeIsObject(fieldsType);
            if (isObject) {
                if (!parent[fieldKey] || parent[fieldKey] < 1) {
                    parent[fieldKey] = parent[fieldKey] || 0;
                    parent[fieldKey]++;
                    return `${field.name} { ${this.getTypeStringFields(fieldsType, parent)} }`;
                }
                else {
                    console.log(`no object: ${field.name} ${fieldsType.name}, ${JSON.stringify(parent)}`);
                }
            }
            else {
                return field.name;
            }
        })
            .filter(item => !!item)
            .join(" ");
    }
}
exports.TypesGenerator = TypesGenerator;
