"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("./utils");
class GraphQlQueryExecutor {
    constructor(request) {
        this.request = request;
    }
    execute(type, items) {
        const data = this.formatQueryData(type, items);
        return this.fetch(data);
    }
    fetch(data) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield utils_1.timeout(1000 * 3, this.request(JSON.stringify(data)));
            return response;
        });
    }
    formatQueryData(type, items) {
        const variables = {};
        let query = type + " queryName";
        const queryParams = {};
        let varCount = 0;
        const keys = Object.keys(items);
        const body = keys
            .map(key => {
            const item = items[key];
            let body = key + ":" + item.name;
            if (item.variables) {
                item.variables.forEach(v => {
                    v.varName = "$input" + varCount;
                    queryParams[v.varName] = v.type || "String!";
                    variables[v.varName.substr(1)] = v.value;
                    varCount++;
                });
                if (varCount > 0) {
                    body +=
                        "(" +
                            item.variables.map(v => v.name + ":" + v.varName).join(", ") +
                            ")";
                }
            }
            if (item.fields) {
                body += "{" + item.fields + "}";
            }
            return body;
        })
            .join(",");
        if (Object.keys(queryParams).length) {
            query +=
                "(" +
                    Object.keys(queryParams)
                        .map(key => key + ":" + queryParams[key])
                        .join(",") +
                    ")";
        }
        query += "{" + body + "}";
        return { query, variables };
    }
}
exports.GraphQlQueryExecutor = GraphQlQueryExecutor;
